/* psdlayerstga by tonic 
 *
 * This is simply the example code for using DrPetter's PSDImage
 * class, with stb_image added for saving each layer as a tga file
 * and some simple options.
 *
 * My home page / blog URL:
 *   http://jet.ro
 *
 * Original URL for PSDImage release:
 *   http://www.ludumdare.com/compo/2008/04/12/psdimage-class/
 * URL for stb_image:
 *   http://nothings.org/stb_image.c
 *
 * Released with the same license than the PSDImage loader:
 * Use it for anything, but do so at your own risk.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include "psdimage.h"
#include "stb_image.c"



int main(int argc, char *argv[])
{
    bool layernum = false;
    bool metatofn = false;
    char *prefix = "";

    printf("psdlayerstga v1.0\n");

    while (argc >= 2 && argv[1][0] == '-')
    {
        if (strcmp(argv[1], "-metatofn") == 0)
        {
            metatofn = true;
            --argc;
            ++argv;
        }
        else if (strcmp(argv[1], "-layernum") == 0)
        {
            layernum = true;
            --argc;
            ++argv;
        }
    }

    if (argc < 2)
    {
        printf("Usage: psdlayers [-metatofn] [-layernum] input.psd [prefix]\n\n");
        printf("Saves each layer as layername.tga file with optional prefix.\n");
        printf("PSD file should be one saved as a \"compatible\" and without\n");
        printf("anything special features such as layer groups.\n\n");
        printf("    -metatofn  use complex output name with prefix, layer\n");
        printf("               number, org. XY offset and layer name.\n\n");
        printf("    -layernum  add layer number to output filenames\n");
        printf("               (used without -metatofn)\n\n");
        return EXIT_SUCCESS;
    }

    if (argc >= 3)
        prefix = argv[2];

    PSDImage *img = new PSDImage();
    if (!img->Load(argv[1])) // load image file and store image data for all layers
    {
        printf("Error loading %s\n", argv[1]);
        return EXIT_FAILURE;
    }

    for (int i = 0; i < img->NumLayers(); ++i) // iterate over all layers (if desired)
    {
        // dimensions of this layer (outer boundaries of image content)
        int width;
        int height;
        // upper-left coordinate of bounding box, use this to place the layer accurately within the image
        int offset_x;
        int offset_y;
        DWORD *imagedata; // pixel array in ARGB format, need not be allocated (will point to stored image data)
        char name[33] = { 0 };
        memcpy(name, img->GetLayer(i)->name, 32);

        imagedata = img->GetLayerData(i, width, height, offset_x, offset_y); // get pointer to layer image, and info

        if (imagedata == NULL)
            printf("Layer %i: FAILED (%s)\n", i, name);
        else
            printf("Layer %i: \"%s\", width:%d, height:%d, x:%d, y:%d%s\n", i, name, width, height, offset_x, offset_y,
                   img->LayerHidden(i) ? ", HIDDEN" : "");

        //if (img->LayerHidden(i))
        //    printf("layer %i (%s) is hidden, but can be used anyway", i, name);

        
        if (width > 0 && height > 0)
        {
            int offs = 0;
            for (int y = 0; y < height; ++y)
            {
                for (int x = 0; x < width; ++x)
                {
                    char tmp, *pb = (char *)&imagedata[offs++];
                    // swap RB
                    tmp = pb[0];
                    pb[0] = pb[2];
                    pb[2] = tmp;
                }
            }
            for (int a = 0; a < 32 && name[a]; ++a)
            {
                int ch = name[a];
                if (!isalnum(ch) && ch != '-' && ch != '_')
                    name[a] = '_';
            }
            char resultFile[256];
            if (metatofn)
                sprintf(resultFile, "%s%d-x%d_y%d_%s.tga", prefix, i, offset_x, offset_y, name);
            else if (layernum)
                sprintf(resultFile, "%s%d-%s.tga", prefix, i, name);
            else
                sprintf(resultFile, "%s%s.tga", prefix, name);
            stbi_write_tga(resultFile, width, height, 4, imagedata);
        }

    }

    delete img; // note that this will free all the image data, so any pointers you might have to layers will be invalid

    // (copy the layer image data if you want to keep it after the PSD object has been destroyed)

    // it's safe to load a new PSD image using the same object, but all previous layer data will of course be lost


    return EXIT_SUCCESS;
}
